import chalk from "chalk";
import { existsSync, linkSync, mkdirSync, readdirSync, statSync, symlinkSync, } from "fs";
import { join, resolve, dirname, basename } from "path";
import { getClient } from "./clients/TorrentClient.js";
import { Action, Decision, InjectionResult, LinkType, SaveResult, } from "./constants.js";
import { logger } from "./logger.js";
import { getRuntimeConfig } from "./runtimeConfig.js";
import { saveTorrentFile } from "./torrent.js";
import { getTag } from "./utils.js";
import { CrossSeedError } from "./errors.js";
export async function validateAction() {
    const { action } = getRuntimeConfig();
    if (action !== Action.INJECT && action !== Action.SAVE) {
        throw new CrossSeedError(`Action method "${action}" is invalid. Allowed choices are "save" and "inject".`);
    }
}
export async function performAction(newMeta, decision, searchee, tracker) {
    const { action, linkDir } = getRuntimeConfig();
    if (searchee.path) {
        if (decision == Decision.MATCH) {
            await linkExact(searchee.path, linkDir);
        }
        else if (decision == Decision.MATCH_SIZE_ONLY) {
            // Size only matching is only supported for single file or
            // single, nested file torrents.
            const candidateParentDir = dirname(newMeta.files[0].path);
            let correctedlinkDir = linkDir;
            // Candidate is single, nested file
            if (candidateParentDir !== ".") {
                mkdirSync(join(linkDir, candidateParentDir), {
                    recursive: true,
                });
                correctedlinkDir = join(linkDir, candidateParentDir);
            }
            linkFile(searchee.path, join(correctedlinkDir, newMeta.files[0].name));
        }
    }
    const styledName = chalk.green.bold(newMeta.name);
    const styledTracker = chalk.bold(tracker);
    if (action === Action.INJECT) {
        const result = await getClient().inject(newMeta, searchee, searchee.path ? linkDir : undefined);
        switch (result) {
            case InjectionResult.SUCCESS:
                logger.info(`Found ${styledName} on ${styledTracker} - injected`);
                break;
            case InjectionResult.ALREADY_EXISTS:
                logger.info(`Found ${styledName} on ${styledTracker} - exists`);
                break;
            case InjectionResult.TORRENT_NOT_COMPLETE:
                logger.warn(`Found ${styledName} on ${styledTracker} - skipping incomplete torrent`);
                break;
            case InjectionResult.FAILURE:
            default:
                logger.error(`Found ${styledName} on ${styledTracker} - failed to inject, saving instead`);
                saveTorrentFile(tracker, getTag(searchee.name), newMeta);
                break;
        }
        return result;
    }
    else {
        saveTorrentFile(tracker, getTag(searchee.name), newMeta);
        logger.info(`Found ${styledName} on ${styledTracker} - saved`);
        return SaveResult.SAVED;
    }
}
export async function performActions(searchee, matches) {
    const results = [];
    for (const { tracker, assessment } of matches) {
        const result = await performAction(assessment.metafile, assessment.decision, searchee, tracker);
        results.push(result);
        if (result === InjectionResult.TORRENT_NOT_COMPLETE)
            break;
    }
    return results;
}
function linkExact(oldPath, newPath) {
    if (!newPath) {
        return;
    }
    if (statSync(oldPath).isFile()) {
        if (!existsSync(join(newPath, basename(oldPath)))) {
            linkFile(oldPath, join(newPath, basename(oldPath)));
        }
        return;
    }
    mkdirSync(join(newPath, basename(oldPath)), { recursive: true });
    readdirSync(oldPath).forEach((file) => {
        linkExact(join(oldPath, file), join(newPath, basename(oldPath)));
    });
}
function linkFile(oldPath, newPath) {
    const { linkType } = getRuntimeConfig();
    try {
        if (linkType === LinkType.HARDLINK) {
            linkSync(oldPath, newPath);
        }
        else {
            // we need to resolve because symlinks are resolved outside
            // the context of cross-seed's working directory
            symlinkSync(oldPath, resolve(newPath));
        }
    }
    catch (e) {
        if (e.code === "EEXIST")
            return;
        throw e;
    }
}
//# sourceMappingURL=action.js.map