import { InjectionResult } from "../constants.js";
import { CrossSeedError } from "../errors.js";
import { Label, logger } from "../logger.js";
import { getRuntimeConfig } from "../runtimeConfig.js";
import { extractCredentialsFromUrl } from "../utils.js";
import fetch, { Headers } from "node-fetch";
var DelugeErrorCode;
(function (DelugeErrorCode) {
    DelugeErrorCode[DelugeErrorCode["NO_AUTH"] = 1] = "NO_AUTH";
    DelugeErrorCode[DelugeErrorCode["BAD_METHOD"] = 2] = "BAD_METHOD";
    DelugeErrorCode[DelugeErrorCode["CALL_ERR"] = 3] = "CALL_ERR";
    DelugeErrorCode[DelugeErrorCode["RPC_FAIL"] = 4] = "RPC_FAIL";
    DelugeErrorCode[DelugeErrorCode["BAD_JSON"] = 5] = "BAD_JSON";
})(DelugeErrorCode || (DelugeErrorCode = {}));
export default class Deluge {
    constructor() {
        this.delugeCookie = null;
        this.delugeLabel = "cross-seed";
    }
    /**
     * validates the login and host for deluge webui
     */
    async validateConfig() {
        await this.authenticate();
        this.isLabelEnabled = await this.labelEnabled();
    }
    /**
     * connects and authenticates to the webui
     */
    async authenticate() {
        const { delugeRpcUrl } = getRuntimeConfig();
        const { href, password } = extractCredentialsFromUrl(delugeRpcUrl).unwrapOrThrow(new CrossSeedError("delugeRpcUrl must be percent-encoded"));
        if (!password) {
            throw new CrossSeedError("You need to define a password in the delugeRpcUrl. (e.g. http://:<PASSWORD>@localhost:8112)");
        }
        const response = await this.call("auth.login", [password], 0);
        if (!response.result) {
            throw new CrossSeedError(`Reached Deluge, but failed to authenticate: ${href}`);
        }
    }
    /**
     * ensures authentication and sends JSON-RPC calls to deluge
     */
    async call(method, params, retries = 1) {
        const { delugeRpcUrl } = getRuntimeConfig();
        const { href } = extractCredentialsFromUrl(delugeRpcUrl).unwrapOrThrow();
        const headers = new Headers({ "Content-Type": "application/json" });
        if (this.delugeCookie)
            headers.set("Cookie", this.delugeCookie);
        let response, json;
        try {
            response = await fetch(href, {
                body: JSON.stringify({
                    ...{ method: method, params: params },
                    id: Math.floor(Math.random() * 0x7fffffff),
                }),
                method: "POST",
                headers: headers,
            });
        }
        catch (networkError) {
            // @ts-expect-error needs es2022 target (tsconfig)
            throw new Error(`Failed to connect to Deluge at ${href}`, {
                cause: networkError,
            });
        }
        try {
            json = await response.json();
        }
        catch (jsonParseError) {
            throw new Error(`Deluge method ${method} response was non-JSON ${jsonParseError}`);
        }
        if (json?.error?.code === DelugeErrorCode.NO_AUTH && retries > 0) {
            this.delugeCookie = null;
            await this.authenticate();
            if (this.delugeCookie) {
                return this.call(method, params, 0);
            }
            else {
                throw new Error("Connection lost with Deluge. Reauthentication failed.");
            }
        }
        this.handleResponseHeaders(response.headers);
        return json;
    }
    /**
     * parses the set-cookie header and updates stored value
     */
    handleResponseHeaders(headers) {
        if (headers.has("Set-Cookie")) {
            this.delugeCookie = headers.get("Set-Cookie").split(";")[0];
        }
    }
    /**
     * checks enabled plugins for "Label"
     * returns true if successful.
     */
    async labelEnabled() {
        const enabledLabels = await this.call("core.get_enabled_plugins", []);
        return enabledLabels?.result?.includes?.("Label");
    }
    /**
     * if Label plugin is loaded, adds (if necessary)
     * and sets the label based on torrent hash.
     */
    async setLabel(infoHash, label) {
        if (this.isLabelEnabled) {
            const setResult = await this.call("label.set_torrent", [
                infoHash,
                label,
            ]);
            if (setResult?.error?.code == DelugeErrorCode.RPC_FAIL) {
                await this.call("label.add", [label]);
                await this.call("label.set_torrent", [infoHash, label]);
            }
        }
    }
    /**
     * injects a torrent into deluge client
     */
    async inject(newTorrent, searchee, path) {
        try {
            let torrentInfo;
            const { duplicateCategories } = getRuntimeConfig();
            if (searchee.infoHash) {
                torrentInfo = await this.getTorrentInfo(searchee);
                if (!torrentInfo.complete) {
                    if (torrentInfo.save_path == "missing" &&
                        !path &&
                        !searchee.path) {
                        return InjectionResult.FAILURE;
                    }
                    return InjectionResult.TORRENT_NOT_COMPLETE;
                }
            }
            const params = this.formatData(`${newTorrent.name}.cross-seed.torrent`, newTorrent.encode().toString("base64"), path ? path : torrentInfo.save_path, !!searchee.infoHash);
            const addResult = await this.call("core.add_torrent_file", params);
            if (addResult?.result) {
                const { dataCategory } = getRuntimeConfig();
                await this.setLabel(newTorrent.infoHash, searchee.path
                    ? dataCategory
                    : torrentInfo.label
                        ? duplicateCategories
                            ? `${torrentInfo.label}.cross-seed`
                            : torrentInfo.label
                        : this.delugeLabel);
                return InjectionResult.SUCCESS;
            }
            else if (addResult?.error?.message?.includes("already")) {
                return InjectionResult.ALREADY_EXISTS;
            }
            else if (addResult?.error?.message) {
                logger.debug({
                    label: Label.DELUGE,
                    message: `Injection failed: ${addResult.error.message}`,
                });
                return InjectionResult.FAILURE;
            }
        }
        catch (injectResult) {
            logger.error({
                label: Label.DELUGE,
                message: `Injection failed: ${injectResult}`,
            });
            logger.debug(injectResult);
            return InjectionResult.FAILURE;
        }
    }
    /**
     * formats the json for rpc calls to inject
     */
    formatData(filename, filedump, path, isTorrent) {
        return [
            filename,
            filedump,
            {
                add_paused: isTorrent ? false : !getRuntimeConfig().skipRecheck,
                seed_mode: isTorrent ? true : getRuntimeConfig().skipRecheck,
                download_location: path,
            },
        ];
    }
    /**
     * returns information needed to complete/validate injection
     */
    async getTorrentInfo(searchee) {
        try {
            let torrentLabel = undefined;
            const params = [
                ["state", "progress", "save_path", "label"],
                { hash: searchee.infoHash },
            ];
            const response = await this.call("web.update_ui", params);
            if (response?.result?.torrents?.[searchee.infoHash] === undefined) {
                throw new Error(`Torrent not found in client (${searchee.infoHash})`);
            }
            if (this.isLabelEnabled &&
                response?.result?.torrents?.[searchee.infoHash]?.label
                    ?.length != 0) {
                torrentLabel =
                    response?.result?.torrents?.[searchee.infoHash]?.label;
            }
            const completedTorrent = response?.result?.torrents?.[searchee.infoHash]?.state ===
                "Seeding" ||
                response?.result?.torrents?.[searchee.infoHash]?.progress ===
                    100;
            return {
                complete: completedTorrent,
                save_path: response?.result?.torrents?.[searchee.infoHash]?.save_path,
                label: torrentLabel,
            };
        }
        catch (e) {
            logger.error({
                label: Label.DELUGE,
                message: `Failed to fetch torrent data: ${searchee.name} - (${searchee.infoHash})`,
            });
            logger.debug(e);
            return { complete: false, save_path: "missing" };
        }
    }
}
//# sourceMappingURL=Deluge.js.map