import { promises as fs } from "fs";
import { dirname, join, resolve, sep } from "path";
import { inspect } from "util";
import xmlrpc from "xmlrpc";
import { InjectionResult } from "../constants.js";
import { CrossSeedError } from "../errors.js";
import { Label, logger } from "../logger.js";
import { Metafile } from "../parseTorrent.js";
import { resultOf, resultOfErr } from "../Result.js";
import { getRuntimeConfig } from "../runtimeConfig.js";
import { extractCredentialsFromUrl, wait } from "../utils.js";
async function createLibTorrentResumeTree(meta, basePath) {
    async function getFileResumeData(file) {
        const filePathWithoutFirstSegment = file.path
            .split(sep)
            .slice(1)
            .join(sep);
        const resolvedFilePath = resolve(basePath, filePathWithoutFirstSegment);
        const fileStat = await fs
            .stat(resolvedFilePath)
            .catch(() => ({ isFile: () => false }));
        if (!fileStat.isFile() || fileStat.size !== file.length) {
            logger.debug({
                label: Label.RTORRENT,
                message: `File ${resolvedFilePath} either doesn't exist or is the wrong size.`,
            });
            return {
                completed: 0,
                mtime: 0,
                priority: 0,
            };
        }
        return {
            completed: Math.ceil(file.length / meta.pieceLength),
            mtime: Math.trunc(fileStat.mtimeMs / 1000),
            priority: 0,
        };
    }
    return {
        bitfield: Math.ceil(meta.length / meta.pieceLength),
        files: await Promise.all(meta.files.map(getFileResumeData)),
    };
}
async function saveWithLibTorrentResume(meta, savePath, basePath) {
    const rawWithLibtorrentResume = {
        ...meta.raw,
        libtorrent_resume: await createLibTorrentResumeTree(meta, basePath),
    };
    await fs.writeFile(savePath, new Metafile(rawWithLibtorrentResume).encode());
}
export default class RTorrent {
    constructor() {
        const { rtorrentRpcUrl } = getRuntimeConfig();
        const { href, username, password } = extractCredentialsFromUrl(rtorrentRpcUrl).unwrapOrThrow(new CrossSeedError("rTorrent url must be percent-encoded"));
        const clientCreator = new URL(href).protocol === "https:"
            ? xmlrpc.createSecureClient
            : xmlrpc.createClient;
        const shouldUseAuth = Boolean(username && password);
        this.client = clientCreator({
            url: href,
            basic_auth: shouldUseAuth
                ? { user: username, pass: password }
                : undefined,
        });
    }
    async methodCallP(method, args) {
        logger.verbose({
            label: Label.RTORRENT,
            message: `Calling method ${method} with params ${inspect(args, {
                depth: null,
                compact: true,
            })}`,
        });
        return new Promise((resolve, reject) => {
            this.client.methodCall(method, args, (err, data) => {
                if (err)
                    return reject(err);
                return resolve(data);
            });
        });
    }
    async checkForInfoHashInClient(infoHash) {
        const downloadList = await this.methodCallP("download_list", []);
        return downloadList.includes(infoHash.toUpperCase());
    }
    async checkOriginalTorrent(searchee) {
        const infoHash = searchee.infoHash.toUpperCase();
        let result;
        try {
            result = await this.methodCallP("system.multicall", [
                [
                    {
                        methodName: "d.directory",
                        params: [infoHash],
                    },
                    {
                        methodName: "d.complete",
                        params: [infoHash],
                    },
                ],
            ]);
        }
        catch (e) {
            logger.debug(e);
            return resultOfErr(InjectionResult.FAILURE);
        }
        // temp diag for #154
        try {
            const [[directoryBase], [isCompleteStr]] = result;
            const isComplete = Boolean(Number(isCompleteStr));
            if (!isComplete) {
                return resultOfErr(InjectionResult.TORRENT_NOT_COMPLETE);
            }
            return resultOf({ directoryBase });
        }
        catch (e) {
            logger.error(e);
            logger.debug("Failure caused by server response below:");
            logger.debug(inspect(result));
            return resultOfErr(InjectionResult.FAILURE);
        }
    }
    async getDownloadLocation(meta, searchee, path) {
        if (path) {
            const basePath = join(path, searchee.name);
            const directoryBase = meta.isSingleFileTorrent ? path : basePath;
            return resultOf({ downloadDir: path, basePath, directoryBase });
        }
        else {
            const result = await this.checkOriginalTorrent(searchee);
            return result.mapOk(({ directoryBase }) => ({
                directoryBase,
                downloadDir: meta.isSingleFileTorrent
                    ? directoryBase
                    : dirname(directoryBase),
                basePath: meta.isSingleFileTorrent
                    ? join(directoryBase, searchee.name)
                    : directoryBase,
            }));
        }
    }
    async validateConfig() {
        const { rtorrentRpcUrl } = getRuntimeConfig();
        // no validation to do
        if (!rtorrentRpcUrl)
            return;
        try {
            await this.methodCallP("download_list", []);
        }
        catch (e) {
            logger.debug(e);
            throw new CrossSeedError(`Failed to reach rTorrent at ${rtorrentRpcUrl}`);
        }
    }
    async inject(meta, searchee, path) {
        const { outputDir } = getRuntimeConfig();
        if (await this.checkForInfoHashInClient(meta.infoHash)) {
            return InjectionResult.ALREADY_EXISTS;
        }
        const result = await this.getDownloadLocation(meta, searchee, path);
        if (result.isErr())
            return result.unwrapErrOrThrow();
        const { directoryBase, basePath } = result.unwrapOrThrow();
        const torrentFilePath = resolve(outputDir, `${meta.name}.tmp.${Date.now()}.torrent`);
        await saveWithLibTorrentResume(meta, torrentFilePath, basePath);
        for (let i = 0; i < 5; i++) {
            try {
                await this.methodCallP("load.start", [
                    "",
                    torrentFilePath,
                    `d.directory_base.set="${directoryBase}"`,
                    `d.custom1.set="cross-seed"`,
                    `d.custom.set=addtime,${Math.round(Date.now() / 1000)}`,
                ]);
                break;
            }
            catch (e) {
                await wait(1000 * Math.pow(2, i));
            }
        }
        for (let i = 0; i < 5; i++) {
            if (await this.checkForInfoHashInClient(meta.infoHash)) {
                setTimeout(() => fs.unlink(torrentFilePath), 1000);
                return InjectionResult.SUCCESS;
            }
            await wait(100 * Math.pow(2, i));
        }
        setTimeout(() => fs.unlink(torrentFilePath), 1000);
        return InjectionResult.FAILURE;
    }
}
//# sourceMappingURL=RTorrent.js.map