import fetch, { Headers } from "node-fetch";
import { InjectionResult } from "../constants.js";
import { CrossSeedError } from "../errors.js";
import { Label, logger } from "../logger.js";
import { resultOf, resultOfErr } from "../Result.js";
import { getRuntimeConfig } from "../runtimeConfig.js";
import { extractCredentialsFromUrl } from "../utils.js";
const XTransmissionSessionId = "X-Transmission-Session-Id";
function doesAlreadyExist(args) {
    return "torrent-duplicate" in args;
}
export default class Transmission {
    async request(method, args = {}, retries = 1) {
        const { transmissionRpcUrl } = getRuntimeConfig();
        const { username, password, href } = extractCredentialsFromUrl(transmissionRpcUrl).unwrapOrThrow(new CrossSeedError("Transmission rpc url must be percent-encoded"));
        const headers = new Headers();
        headers.set("Content-Type", "application/json");
        if (this.xTransmissionSessionId) {
            headers.set(XTransmissionSessionId, this.xTransmissionSessionId);
        }
        if (username && password) {
            const credentials = Buffer.from(`${username}:${password}`).toString("base64");
            headers.set("Authorization", `Basic ${credentials}`);
        }
        const response = await fetch(href, {
            method: "POST",
            body: JSON.stringify({ method, arguments: args }),
            headers,
        });
        if (response.status === 409) {
            this.xTransmissionSessionId = response.headers.get(XTransmissionSessionId);
            return this.request(method, args, retries - 1);
        }
        try {
            const responseBody = (await response.json());
            if (responseBody.result === "success" ||
                responseBody.result === "duplicate torrent" // slight hack but best solution for now
            ) {
                return responseBody.arguments;
            }
            else {
                throw new Error(`Transmission responded with error: "${responseBody.result}"`);
            }
        }
        catch (e) {
            if (e instanceof TypeError) {
                logger.error({
                    label: Label.TRANSMISSION,
                    message: `Transmission returned non-JSON response`,
                });
                logger.debug({
                    label: Label.TRANSMISSION,
                    message: response,
                });
            }
            else {
                logger.error({
                    label: Label.TRANSMISSION,
                    message: `Transmission responded with an error`,
                });
                logger.debug(e);
            }
            throw e;
        }
    }
    async validateConfig() {
        try {
            await this.request("session-get");
        }
        catch (e) {
            const { transmissionRpcUrl } = getRuntimeConfig();
            throw new CrossSeedError(`Failed to reach Transmission at ${transmissionRpcUrl}`);
        }
    }
    async checkOriginalTorrent(searchee) {
        let queryResponse;
        try {
            queryResponse = await this.request("torrent-get", {
                fields: ["downloadDir", "percentDone"],
                ids: [searchee.infoHash],
            });
        }
        catch (e) {
            return resultOfErr(InjectionResult.FAILURE);
        }
        if (queryResponse.torrents.length === 0) {
            return resultOfErr(InjectionResult.FAILURE);
        }
        const [{ downloadDir, percentDone }] = queryResponse.torrents;
        if (percentDone < 1) {
            return resultOfErr(InjectionResult.TORRENT_NOT_COMPLETE);
        }
        return resultOf({ downloadDir });
    }
    async inject(newTorrent, searchee, path) {
        let downloadDir;
        if (path) {
            downloadDir = path;
        }
        else {
            const result = await this.checkOriginalTorrent(searchee);
            if (result.isErr()) {
                return result.unwrapErrOrThrow();
            }
            downloadDir = result.unwrapOrThrow().downloadDir;
        }
        let addResponse;
        try {
            addResponse = await this.request("torrent-add", {
                "download-dir": downloadDir,
                metainfo: newTorrent.encode().toString("base64"),
                paused: false,
                labels: ["cross-seed"],
            });
        }
        catch (e) {
            return InjectionResult.FAILURE;
        }
        if (doesAlreadyExist(addResponse)) {
            return InjectionResult.ALREADY_EXISTS;
        }
        return InjectionResult.SUCCESS;
    }
}
//# sourceMappingURL=Transmission.js.map