import { join } from "path";
import winston from "winston";
import { appDir, createAppDir } from "./configuration.js";
import { getRuntimeConfig } from "./runtimeConfig.js";
import DailyRotateFile from "winston-daily-rotate-file";
export var Label;
(function (Label) {
    Label["QBITTORRENT"] = "qbittorrent";
    Label["RTORRENT"] = "rtorrent";
    Label["TRANSMISSION"] = "transmission";
    Label["DELUGE"] = "deluge";
    Label["DECIDE"] = "decide";
    Label["PREFILTER"] = "prefilter";
    Label["CONFIGDUMP"] = "configdump";
    Label["TORZNAB"] = "torznab";
    Label["SERVER"] = "server";
    Label["STARTUP"] = "startup";
    Label["SCHEDULER"] = "scheduler";
    Label["SEARCH"] = "search";
    Label["RSS"] = "rss";
    Label["PERF"] = "perf";
    Label["REVERSE_LOOKUP"] = "reverselookup";
})(Label = Label || (Label = {}));
export let logger;
const redactionMsg = "[REDACTED]";
function redactUrlPassword(message, urlStr) {
    let url;
    try {
        url = new URL(urlStr);
        if (url.password) {
            const urlDecodedPassword = decodeURIComponent(url.password);
            const urlEncodedPassword = encodeURIComponent(url.password);
            message = message.split(url.password).join(redactionMsg);
            message = message.split(urlDecodedPassword).join(redactionMsg);
            message = message.split(urlEncodedPassword).join(redactionMsg);
        }
    }
    catch (e) {
        // do nothing
    }
    return message;
}
function redactMessage(message) {
    if (typeof message !== "string") {
        return message;
    }
    const runtimeConfig = getRuntimeConfig();
    let ret = message;
    // redact torznab api keys
    ret = ret.replace(/apikey=[a-zA-Z0-9]+/g, `apikey=${redactionMsg}`);
    ret = ret.replace(/\/notification\/crossSeed\/[a-zA-Z-0-9_-]+/g, `/notification/crossSeed/${redactionMsg}`);
    for (const [key, urlStr] of Object.entries(runtimeConfig)) {
        if (key.endsWith("Url") && urlStr) {
            ret = redactUrlPassword(ret, urlStr);
        }
    }
    return ret;
}
const logOnceCache = [];
export function logOnce(cacheKey, cb) {
    if (!logOnceCache.includes(cacheKey)) {
        logOnceCache.push(cacheKey);
        cb();
    }
}
export function initializeLogger() {
    createAppDir();
    logger = winston.createLogger({
        level: "info",
        format: winston.format.combine(winston.format.timestamp({
            format: "YYYY-MM-DD HH:mm:ss",
        }), winston.format.errors({ stack: true }), winston.format.splat(), winston.format.colorize(), winston.format.printf(({ level, message, label, timestamp }) => {
            return `${timestamp} ${level}: ${label ? `[${label}] ` : ""}${redactMessage(message)}`;
        })),
        transports: [
            new DailyRotateFile({
                filename: "error.%DATE%.log",
                createSymlink: true,
                symlinkName: "error.current.log",
                dirname: join(appDir(), "logs"),
                maxFiles: "14d",
                level: "error",
            }),
            new DailyRotateFile({
                filename: "info.%DATE%.log",
                createSymlink: true,
                symlinkName: "info.current.log",
                dirname: join(appDir(), "logs"),
                maxFiles: "14d",
            }),
            new DailyRotateFile({
                filename: "verbose.%DATE%.log",
                createSymlink: true,
                symlinkName: "verbose.current.log",
                dirname: join(appDir(), "logs"),
                maxFiles: "14d",
                level: "silly",
            }),
            new winston.transports.Console({
                level: getRuntimeConfig().verbose ? "silly" : "info",
                format: winston.format.combine(winston.format.errors({ stack: true }), winston.format.splat(), winston.format.colorize(), winston.format.printf(({ level, message, label }) => {
                    return `${level}: ${label ? `[${label}] ` : ""}${redactMessage(message)}`;
                })),
            }),
        ],
    });
}
//# sourceMappingURL=logger.js.map